`timescale 1ns / 1ps

// VERSION 1.1

/* A parameterized bus-width, 16-to-1 mux. */
module mux_Nbit_16to1(out, a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p, sel);
   parameter z = 1;
   
   output [z-1:0] out;
   input [z-1:0]  a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p;
   input [3:0]    sel;

   assign         out = sel == 4'b0000 ? a : 
                        sel == 4'b0001 ? b :
                        sel == 4'b0010 ? c : 
                        sel == 4'b0011 ? d :
                        sel == 4'b0100 ? e :
                        sel == 4'b0101 ? f :
                        sel == 4'b0110 ? g :
                        sel == 4'b0111 ? h :
                        sel == 4'b1000 ? i :
                        sel == 4'b1001 ? j :
                        sel == 4'b1010 ? k :
                        sel == 4'b1011 ? l :
                        sel == 4'b1100 ? m :
                        sel == 4'b1101 ? n :
                        sel == 4'b1110 ? o :
                        sel == 4'b1111 ? p : {{z}{1'b0}} /*garbage*/;
endmodule // mux_Nbit_16to1


/* A parameterized bus-width, 8-to-1 mux. */
module mux_Nbit_8to1(out, a, b, c, d, e, f, g, h, sel);
   parameter n = 1;
   
   output [n-1:0] out;
   input [n-1:0]  a, b, c, d, e, f, g, h;
   input [2:0]    sel;

   assign         out = sel == 3'b000 ? a : 
                        sel == 3'b001 ? b :
                        sel == 3'b010 ? c : 
                        sel == 3'b011 ? d :
                        sel == 3'b100 ? e :
                        sel == 3'b101 ? f :
                        sel == 3'b110 ? g :
                        sel == 3'b111 ? h : {{n}{1'b0}} /*garbage*/;
endmodule // mux_Nbit_8to1


/* A parameterized bus-width, 4-to-1 mux. 
 sel==00 -> a
 sel==01 -> b
 sel==10 -> c
 sel==11 -> d 
*/
module mux_Nbit_4to1(out, a, b, c, d, sel);
   parameter n = 1;
   
   output [n-1:0] out;
   input [n-1:0]  a, b, c, d;
   input [1:0]    sel;

   assign         out = sel == 2'b00 ? a : 
                        sel == 2'b01 ? b :
                        sel == 2'b10 ? c : 
                        sel == 2'b11 ? d : {{n}{1'b0}} /*garbage*/;
endmodule

/* A parameterized bus-width, 2-to-1 mux. 
 *  sel=0 selects `a', 
 *  sel=1 selects `b'.
 */
module mux_Nbit_2to1(out, a, b, sel);
   parameter n = 1;
   
   output [n-1:0] out;
   input [n-1:0]  a, b;
   input          sel;

   assign         out = sel == 1'b0 ? a : 
                        sel == 1'b1 ? b : {{n}{1'b0}} /*garbage*/;
endmodule